﻿using UnityEngine;
using System.Collections;

[RequireComponent(typeof(BoxCollider2D))]
public class PlayerMover : MonoBehaviour {
	private Vector2 size;
	
	private bool moving_horizontally = false;
	private bool moving_vertically = false;
	
	private int stunned = 0;
	private bool crazed = false;
	
	public int id;
	public PlayerMovementType movementType;
	
	public float movementTime;
	
	private bool can_move = true;
	private bool can_teleport = true;
	
	private Vector2 movement_start_position;
	
	void Start() {
		size = GetComponent<BoxCollider2D>().size;
	}
	
	private Vector2 getInput() {
		Vector2 movement = Vector2.zero;
		
		float horizontal_axis = Input.GetAxisRaw ("Horizontal");
		float vertical_axis   = Input.GetAxisRaw ("Vertical");
		
		if (horizontal_axis != 0 && moving_horizontally == false) {
			moving_horizontally = true;			
			movement.x = horizontal_axis * size.x * (movementType == PlayerMovementType.Single ? 1 : 2);
		}
		else if (horizontal_axis == 0) {
			moving_horizontally = false;
		}
		
		if (vertical_axis != 0 && moving_vertically == false) {
			moving_vertically = true;
			movement.y = vertical_axis   * size.y * (movementType == PlayerMovementType.Single ? 1 : 2);
		}
		else if (vertical_axis == 0) {
			moving_vertically = false;
		}
		
		return movement;
	}
	
	void Update () {
		if (!can_move) return;
	
		Vector2 movement = getInput();

		if (movement.magnitude > 0) {
			if (stunned > 0) {
				movement = Vector2.zero;
				//stunned--;
			}
			if (crazed) {
				movement = -movement;
				//crazed--;
			}
		}
		
		movement = BlockDetect(movement);
		
		if (movement.magnitude > 0) {
			SetAnimationDirection(movement);
			StartCoroutine ("LerpMovement", movement);
		}
	}
	
	private void SetAnimationDirection(Vector2 movement) {
		int direction;
		if (movement.y == 0) {
			direction = (movement.x > 0) ? 1 : 3;
		}
		else {
			direction = (movement.y > 0) ? 2 : 0;
		}
		GetComponent<Animator>().SetInteger("direction", direction);
	}
	
	private Vector2 BlockDetect(Vector2 movement) {
		int layer_mask = 1 << LayerMask.NameToLayer("Block");
		RaycastHit2D hit;
		if (hit = Physics2D.Raycast(transform.position, movement, movement.magnitude, layer_mask)) {		
			switch(movementType) {
				
			case PlayerMovementType.Single:
				movement = Vector2.zero;
				break;
				
			case PlayerMovementType.Double:
				if (hit.distance > size.x) {
					movement = movement / 2;
				}
				else if (hit.distance < size.x) {
					movement = Vector2.zero;
				}
				break;
				
			case PlayerMovementType.Jump:
				if (hit.distance > size.x) {
					movement = movement / 2;
				}
				else if (hit.distance < size.x) {
					if (Physics2D.Raycast(transform.position + (Vector3)movement, movement, size.x * .4f, layer_mask))
						movement = Vector2.zero;
				}
				break;
			}
		}
		
		return movement;
	}
	
	public void setStun(int value) {
		StartCoroutine("stun", value);
	}
	IEnumerator stun(int value) {
		stunned = 1;
		GetComponent<Animator>().SetBool("eating_good", true);
		GetComponent<PlayerAudioManager>().PlaySound(PlayerSounds.EatSound);
		
		float elapsed_time = 0;
		while (elapsed_time < value) {		
			transform.GetChild(0).transform.localScale = new Vector3(Mathf.Lerp (1, 0, elapsed_time / value), 0, 0);
			elapsed_time += Time.deltaTime;
			yield return null;
		}

		GetComponent<Animator>().SetBool("eating_good", false);
		stunned = 0;
	}
	public void setCrazed() {
		GetComponent<PlayerAudioManager>().PlaySound(PlayerSounds.DrunkSound);
		crazed = true;
	}
	
	private IEnumerator LerpMovement(Vector2 movement) {
		if (movement.magnitude == 0) yield break;
	
		movement_start_position = transform.position;
		startMoving();
		
		float elapsed_time = 0;
		
		while (elapsed_time < movementTime) {
			transform.position = Vector2.Lerp(movement_start_position, movement_start_position + movement, Mathf.Sqrt(elapsed_time / movementTime));
			elapsed_time += Time.deltaTime;
			yield return null;
		}
		
		transform.position = movement_start_position + movement;		
		stopMoving();

		yield break;
	}
	
	private void startMoving() {
		can_move = false;
		GetComponent<Animator>().SetBool("walking", true);
		GetComponent<PlayerAudioManager>().PlaySound(
			(movementType == PlayerMovementType.Jump) ? PlayerSounds.JumpSound : PlayerSounds.WalkSound
		);
		if (movementType == PlayerMovementType.Jump) GetComponent <BoxCollider2D>().enabled = false;
	}
	
	private void stopMoving() {
		can_move = true;
		can_teleport = true;
		GetComponent<Animator>().SetBool("walking", false);
		if (movementType == PlayerMovementType.Jump) GetComponent <BoxCollider2D>().enabled = true;
	}
	
	public void Port(Vector3 destination)
	{
		if (can_teleport) {
			can_teleport = false;
			GetComponent<PlayerAudioManager>().PlaySound(PlayerSounds.TeleportSound);
			movement_start_position = destination;
		}
	}
	
	public int ID { get { return id; } }
}


public enum PlayerMovementType {
	Single,
	Double,
	Jump
}
