﻿using UnityEngine;
using System.Collections;

[RequireComponent (typeof(BoxCollider2D))]

public class RotatingHammer : MonoBehaviour {

	public float DelayBetweenRotations;
	public float RotationTime;
	
	private bool can_hit;
	private Vector2 direction = new Vector2(1,0);
	private Quaternion rotation = new Quaternion();
	private float size;
	
	private bool stopped = false;
	
	void Start() {
		size = GetComponent<BoxCollider2D>().size.x;		
		StartCoroutine("Rotate");
	}
	
	// Update is called once per frame
	void Update () {
		if (can_hit) {
			int layer_mask = 1 << LayerMask.NameToLayer("Player");
			
			RaycastHit2D hit;
			if (hit = Physics2D.Raycast(transform.position, rotation * direction, size, layer_mask)) {
				hit.collider.gameObject.SendMessage("die", DeathType.Squished);
				stopped = true;
				GetComponent<Animator>().SetBool ("hit", true);
			}
		}
	}
	
	IEnumerator Rotate() {
		while (true) {
			// Waiting phase - it can kill you
			can_hit = true;
			if (stopped) yield break;
			
			yield return new WaitForSeconds(DelayBetweenRotations);
			
			if (stopped) yield break;
		
			// -------------------------------------------------------------------
		
			// Rotating phase - it cannot kill you until it stops moving. 
			can_hit = false;
			
			float elapsed_time = 0;
			
			float initial_angle = rotation.eulerAngles.z;
			
			while (elapsed_time < RotationTime) {
				elapsed_time += Time.deltaTime;
				
				rotation.eulerAngles = new Vector3(0,0,Mathf.Lerp (initial_angle, initial_angle + 90, elapsed_time / RotationTime));
				
				yield return null;
			}
			setAnimation();
		}
	}
	
	void setAnimation() {
		float animation_direction = rotation.eulerAngles.z / 90;
		animation_direction %= 4;
		GetComponent<Animator>().SetInteger("Direction", (int)animation_direction);
	}
}
